#!/usr/bin/env python3
from ovos_bus_client import MessageBusClient
from ovos_utils import wait_for_exit_signal

from ovos_plugin_common_play.ocp import OCP
from ovos_plugin_common_play.ocp.constants import OCP_ID


def main():
    """
    Console script entrypoint
    USAGE: ovos-ocp-standalone
    """
    bus = MessageBusClient()
    bus.run_in_thread()
    bus.connected_event.wait()
    config = {"mode": "external"}

    try:
        ocp = OCP(bus=bus, skill_id=OCP_ID, settings=config)
    except ImportError:
        print("OCP is not available")
        ocp = None

    wait_for_exit_signal()

    if ocp:
        ocp.shutdown()


if __name__ == "__main__":
    main()



import time
from threading import RLock

from ovos_bus_client.message import Message
from ovos_utils.log import LOG
from ovos_utils.ocp import MediaType

from ovos_plugin_common_play.ocp.base import OCPAbstractComponent
from ovos_plugin_common_play.ocp.media import Playlist


class OCPSearch(OCPAbstractComponent):
    def __init__(self, player=None):  # OCPMediaPlayer
        super(OCPSearch, self).__init__(player)
        self.search_playlist = Playlist()
        self.ocp_skills = {}
        self.featured_skills = {}
        self.search_lock = RLock()
        if player:
            self.bind(player)

    def bind(self, player):  # OCPMediaPlayer
        self._player = player
        self.add_event("ovos.common_play.skills.detach",
                       self.handle_ocp_skill_detach)
        self.add_event("ovos.common_play.announce",
                       self.handle_skill_announce)
        self.add_event("ovos.common_play.search.start",
                       self.handle_search_start)

    def handle_search_start(self, message):
        self.gui.notify_search_status("Searching...")

    def shutdown(self):
        self.remove_event("ovos.common_play.announce")
        self.remove_event("ovos.common_play.skills.detach")

    def handle_skill_announce(self, message):
        skill_id = message.data.get("skill_id")
        skill_name = message.data.get("skill_name") or skill_id
        img = message.data.get("thumbnail")
        has_featured = bool(message.data.get("featured_tracks"))
        media_type = message.data.get("media_type") or [MediaType.GENERIC]

        if skill_id not in self.ocp_skills:
            LOG.debug(f"Registered {skill_id}")
            self.ocp_skills[skill_id] = []

        if has_featured:
            LOG.debug(f"Found skill with featured media: {skill_id}")
            self.featured_skills[skill_id] = {
                "skill_id": skill_id,
                "skill_name": skill_name,
                "thumbnail": img,
                "media_type": media_type
            }

    def handle_ocp_skill_detach(self, message):
        skill_id = message.data["skill_id"]
        if skill_id in self.ocp_skills:
            self.ocp_skills.pop(skill_id)
        if skill_id in self.featured_skills:
            self.featured_skills.pop(skill_id)

    def get_featured_skills(self, adult=False):
        # trigger a presence announcement from all loaded ocp skills
        self.bus.emit(Message("ovos.common_play.skills.get"))
        time.sleep(0.2)
        skills = list(self.featured_skills.values())
        if adult:
            return skills
        return [s for s in skills
                if MediaType.ADULT not in s["media_type"] and
                MediaType.HENTAI not in s["media_type"]]



import shutil
from os import makedirs
from os.path import expanduser, isfile, join, dirname, exists
from ovos_config import Configuration
from ovos_utils.log import LOG
from ovos_plugin_manager.ocp import load_stream_extractors, available_extractors
from functools import wraps


def validate_message_context(message, native_sources=None):
    destination = message.context.get("destination")
    if destination:
        native_sources = native_sources or Configuration()["Audio"].get(
            "native_sources", ["debug_cli", "audio"]) or []
        if any(s in destination for s in native_sources):
            # request from device
            return True
        # external request, do not handle
        return False
    # broadcast for everyone
    return True


def require_native_source():

    def _decorator(func):
        @wraps(func)
        def func_wrapper(self, message=None):
            validated = message is None or \
                        not self.validate_source or \
                        validate_message_context(message, self.native_sources)
            if validated:
                return func(self, message)
            LOG.debug("ignoring OCP bus message, not from a native audio source")
            return None

        return func_wrapper

    return _decorator


def ocp_plugins():
    return load_stream_extractors()


def is_qtav_available():
    return exists("/usr/include/qt/QtAV") or \
        exists("/usr/lib/qt/qml/QtAV") or \
        exists("/usr/lib/libQtAV.so")


def extract_metadata(uri):
    # backwards compat
    from ovos_ocp_files_plugin.plugin import OCPFilesMetadataExtractor
    return OCPFilesMetadataExtractor.extract_metadata(uri)


def create_desktop_file():
    res = join(dirname(__file__), "res", "desktop")
    desktop_path = expanduser("~/.local/share/applications")
    icon_path = expanduser("~/.local/share/icons")
    makedirs(desktop_path, exist_ok=True)
    makedirs(icon_path, exist_ok=True)

    src_desktop = join(res, "OCP.desktop")
    dst_desktop = join(desktop_path, "OCP.desktop")
    if not isfile(dst_desktop):
        shutil.copy(src_desktop, dst_desktop)

    src_icon = join(res, "OCP.png")
    dst_icon = join(icon_path, "OCP.png")
    if not isfile(dst_icon):
        shutil.copy(src_icon, dst_icon)
